import luxe.States;
import luxe.Color;
import luxe.Vector;
import luxe.Input;
import luxe.Text;

import luxe.Parcel;
import luxe.ParcelProgress;
import luxe.resource.Resource;

import luxe.utils.Maths;
import luxe.utils.Random;

import mint.Control;
import mint.types.Types;
import mint.render.luxe.LuxeMintRender;
import mint.render.luxe.Convert;
import mint.layout.margins.Margins;

/* Created for Ludum Dare 33 - Theme: "You are the Monster"
	by Deity (  http://imdeity.com/ludum/33/ )
*/

class Main extends luxe.Game {

	var state   : States;

	var current : Int = 0;
	var count : Int = 0;

	public static var disp : Text;
	public static var canvas: mint.Canvas;
	public static var rendering: LuxeMintRender;
	public static var layout: Margins;

	var canvas_debug : Text;
	var debug : Bool = false;

	public static var rng 			: Random;
	public static var world			: World;
	public static var player 		: Player;

	override function config(config:luxe.AppConfig) {

		// Capture space key on web (scrolls page by default)
		config.web.prevent_default_keys = [Key.space, Key.left, Key.right, Key.up, Key.down];

		return config;

	} //config


	// On ready
	override function ready() {

		trace("* Main ready");

		rng 		= new Random( Luxe.time );

		initLoad();

	}


	function initLoad() {
		
		// Fetch and load the Parcel JSON (returns a promise)
		Luxe.resources.load_json('assets/parcel.json').then( function( json:JSONResource ) {
			
			// Success on parcel json load
			trace('* M - Parcel json loaded');
			
			var parcel = new Parcel();
			parcel.from_json( json.asset.json );
			
			// Simpe progress bar
			new ParcelProgress({
				parcel      : parcel,
				oncomplete  : onLoaded, 	// Callback to fire when done
				background  : new Color(0, 0, 0, 0),
				bar         : new Color().rgb(0xD64927),
				bar_border  : new Color().rgb(0xDEDEDE),
				fade_in     : false,
				fade_out    : true
			});

			// Go!
			parcel.load();
			
		}, function( err:Dynamic ) {
			// Error handler
			trace('* M - Parcel json load FAIL');
			trace( err );
		});
	}


	// After assets loaded
	function onLoaded( p:Parcel ) {

		Luxe.renderer.clear_color.rgb(0x526631);

		rendering = new LuxeMintRender();
		layout = new Margins();

		canvas = new mint.Canvas({
			name:'canvas',
			rendering: rendering,
			options: { color:new Color(1,1,1,0.0) },
			x: 0, y:0, w: 960, h: 640
		});

		disp = new Text({
			name:'display.text',
			pos: new Vector(Luxe.screen.w-10, Luxe.screen.h-10),
			align: luxe.TextAlign.right,
			align_vertical: luxe.TextAlign.bottom,
			point_size: 13,
			text: 'Theme: "You are the Monster"\nby ImDeity for LD33'
		});

		/*
		canvas_debug = new Text({
			name:'debug.text',
			text: 'debug:  (${Luxe.snow.os} / ${Luxe.snow.platform})',
			point_size: 14,
			pos: new Vector(950, 10),
			align: right,
			depth: 999,
			color: new Color()
		});
		*/

		state = new States();

		state.add( new PlayState( 'play' ) );
/*		state.add( new tests.KitchenSink({ name:'state0' }) );
		state.add( new tests.Scrolling({ name:'state1' }) );
		state.add( new tests.Depth({ name:'state2' }) );*/

		count = Lambda.count( state._states );

		// state.set( 'state0' );
		state.set( 'play' );

	} //ready


	function change(to:String) {

		disp.text = '';

		canvas.destroy_children();
		state.set(to);

	} //change
	

	override function onrender() {

		if (canvas == null) { return; }

		canvas.render();

		if(debug) {
			for(c in canvas.children) {
				drawc(c);
			}
		}

	} //onrender


	override function update(dt:Float) {

		if (canvas == null) { return; }

		canvas.update(dt);

	} //update


	override function onmousemove(e) {

		if (canvas == null || canvas_debug == null) { return; }

		canvas.mousemove( Convert.mouse_event(e) );

		var s = 'debug:  (${Luxe.snow.os} / ${Luxe.snow.platform})\n';

		s += 'canvas nodes: ' + (canvas != null ? '${canvas.nodes}' : 'none');
		s += '\n';
		s += 'focused: ' + (canvas.focused != null ? '${canvas.focused.name} [${canvas.focused.nodes}]' : 'none');
		s += (canvas.focused != null ? ' / depth: '+canvas.focused.depth : '');
		s += '\n';
		s += 'modal: ' + (canvas.modal != null ?  canvas.modal.name : 'none');
		s += '\n';
		s += 'dragged: ' + (canvas.dragged != null ? canvas.dragged.name : 'none');
		s += '\n\n';

		canvas_debug.text = s;

	} //onmousemove


	override function onmousewheel(e) {
		if (canvas == null) { return; }

		canvas.mousewheel( Convert.mouse_event(e) );
	}


	override function onmouseup(e) {
		if (canvas == null) { return; }

		canvas.mouseup( Convert.mouse_event(e) );
	}


	override function onmousedown(e) {
		if (canvas == null) { return; }

		canvas.mousedown( Convert.mouse_event(e) );
	}


	override function onkeydown(e:luxe.Input.KeyEvent) {
		if (canvas == null) { return; }

		canvas.keydown( Convert.key_event(e) );
	}


	override function ontextinput(e:luxe.Input.TextEvent) {
		if (canvas == null) { return; }

		canvas.textinput( Convert.text_event(e) );
	}


	override function onkeyup(e:luxe.Input.KeyEvent) {
		/*
		if (canvas == null) { return; }

		if(e.keycode == Key.key_d && e.mod.ctrl) { debug = !debug; trace('debug: $debug'); }
		if(e.keycode == Key.key_v && e.mod.ctrl) canvas.visible = !canvas.visible;

		if(e.keycode == Key.key_9) {
			current--;
			if(current < 0) current = count-1;
			change('state$current');
		}

		if(e.keycode == Key.key_0) {
			current++;
			if(current >= count) current = 0;
			change('state$current');
		}

		canvas.keyup( Convert.key_event(e) );
		*/

		if(e.keycode == Key.escape) {
			Luxe.shutdown();
		}

	} //onkeyup


	function drawc(control:Control) {

		if(!control.visible) return;

		Luxe.draw.rectangle({
			depth: 1000,
			x: control.x,
			y: control.y,
			w: control.w,
			h: control.h,
			color: new Color(1,0,0,0.5),
			immediate: true
		});

		for(c in control.children) {
			drawc(c);
		}

	} //drawc


	public static function playClick( _randomPitch : Bool = true ) {
		/*// Play sound with random pitch
		if (_randomPitch) {
			Luxe.audio.pitch('click', Main.rng.float(0.75, 1.25));
		}*/
		playSound('click', _randomPitch);
	}
	

	public static function playSound( _soundName : String = 'click', _randomPitch : Bool = false ) {

		// Sounds and how many variations of each there are
		var soundList : Map<String, Int> = [
			'click'			=> 1,
			'blip'			=> 1,
			'coin'			=> 1,
			'hit'			=> 1,
			'pistol'		=> 1,
			'throw'			=> 1			
		];

		// Play sound with random pitch
		if (_randomPitch) {
			Luxe.audio.pitch( _soundName, Main.rng.float(0.8, 1.2) );
		}

		// Randomly select from sound list variations if theres more than 1
		if (soundList.exists( _soundName )) {
			if ( soundList.get( _soundName ) > 1 ) {
				_soundName = _soundName + rng.int(1, soundList.get( _soundName ) + 1 );
			}
		}

		Luxe.audio.play( _soundName );
	}

} //Main

