package;

import luxe.Entity;
import luxe.Sprite;
import luxe.Vector;
import luxe.Color;
import luxe.tween.Actuate;
import luxe.Particles;

import luxe.components.sprite.SpriteAnimation;

import phoenix.Texture.FilterType;

/**
 * Skeleton object, holds sprite and props
 * @author DevZero
 */
class Skeleton extends Sprite {

	public var speed 		: Float = 40;

	var anim 				: SpriteAnimation;

	// Instantiation
	public function new() {

		trace("* new Skeleton");

		// Give player a visual
		super({
			name: 'skeleton',
			name_unique: true,
			pos: new Vector( 160, 160),
			size: new Vector(30, 42),
			texture: Luxe.resources.texture('assets/sprites/Skeleton.png'),
			depth: 3
		});

		scale = new Vector(0.9, 0.9);

		initSprite();

	}

	
	// Init
	override function init() {

		trace('* Skeleton Init');

	}

	
	public function moveTo( inEntity : Entity, removeOnComplete : Bool = false  ) {

		if ( inEntity == null || inEntity.pos == null) { trace('* moveTo() Invalid input'); }

		var inPos : Vector = inEntity.pos.clone();
		var dist : Float = inPos.clone().subtract(pos).length;
		var tweenTime : Float = dist / speed;

		// trace('* Skel tweenTime ${tweenTime}');

		// Flip facing direction based on movement direction
		flipx = (inPos.x > pos.x) ? true : false;

		Actuate.tween( pos, tweenTime, {
			x: inPos.x,
			y: inPos.y
		})
		.timescale(true)
		.ease( luxe.tween.easing.Sine.easeInOut )
		.onComplete( function() {

			// Scale out and remove
			if (removeOnComplete) {
				Actuate.tween( scale, 0.7, {
					x: 0.01,
					y: 0.01,
				})
				.timescale(true)
				.ease( luxe.tween.easing.Quart.easeIn )
				.onComplete( function() {
					destroy();
				});
			}
		});
	}

	
	override function ondestroy() {
		super.ondestroy();
	}


	function initSprite() {

		anim = new SpriteAnimation({ name: 'skeletonanim' });
		add(anim);

		
		var animation_json = '
			{
				"idle" : {
					"frame_size":{ "x":"30", "y":"42" },
					"frameset": ["1-6"],
					"events" : [],
					"pingpong": "true",
					"loop": "true",
					"speed": "12"
				},
				"walk" : {
					"frame_size":{ "x":"34", "y":"36" },
					"frameset": ["1-12"],
					"events" : [{"frame":8, "event":"foot.1"}, {"frame":1, "event":"foot.2"}, { "frame": 6 }],
					"pingpong":"false",
					"loop": "true",
					"speed": "18"
				},
				"walk_glitch" : {
					"frame_sources":[{
						"frame":8, "x":0, "y":0, "w":96, "h":96
					}],
					"frame_size":{ "x":"48", "y":"48" },
					"frameset": ["1-8","9","10","hold 10","11 hold 5", "12"],
					"pingpong":"false",
					"loop": "true",
					"speed": "18"
				}
			}
		';

		anim.add_from_json( animation_json );

		anim.animation = 'idle';
		anim.play();

		// Do thing on sprite anim events
		/*sprite.events.listen('foot.1', function(e){
            Luxe.audio.play('step1'); //:todo:
        });
		*/


	}

}