package;

import luxe.Entity;
import luxe.Vector;
import luxe.Color;
import luxe.Visual;
import luxe.Sprite;
import luxe.Rectangle;
import luxe.Text;
import luxe.Input;

/**
 * ...
 * @author DevZero
 */
class World extends Entity {

	public static var wWidth 	: Int = 960;
	public static var wHeight 	: Int = 640;
	public static var wMargin 	: Int = 30;

	public var outposts	: Array<Outpost>;
	public var towns 	: Array<Town>;
	public var persons 	: Array<Person>;

	var startTowns 		: Int = 4;

	var startPopMin 	: Int = 5;
	var startPopMax 	: Int = 20;

	public var score 		: Int = 0;
	var _scoreText			: Text;

	public function new() {

		trace("* World new");
		
		super({ name: 'World' });

		outposts 	= new Array<Outpost>();
		towns 		= new Array<Town>();
		persons 	= new Array<Person>();

	}

	// Init
	override function init() {
		trace('* World Init');
		
		score = 0;
		_scoreText = new Text({
			name:'score',
			text: 'Score: 0',
			point_size: 18,
			pos: new Vector(Luxe.screen.mid.x, 10),
			align: center,
			align_vertical: top,
			depth: 999,
			color: new Color().rgb(0xF3F781)
		});


		initTowns();
		initOutposts();

		draw();
	}

	public function incrScore( inVal : Int ) {

		setScore( score + inVal );
	}

	public function setScore( inVal : Int ) {

		score = inVal;
		_scoreText.text = 'Score: ' + score;

	}


	function initOutposts() {
		trace('* World initOutposts');

		var tempPos : Vector = new Vector();
		while (outposts.length < 1) {

			tempPos = randomPosition();

			if (checkForOverlap(tempPos, 140)) {
				// Overlapping, try again
				continue;
			}

			var anOutpost : Outpost = new Outpost();
			anOutpost.pos = tempPos.clone();

			outposts.push( anOutpost );
		}
	}
	

	function initTowns() {
		trace('* World initTowns');

		var tempPos : Vector = new Vector();
		while (towns.length < startTowns) {

			tempPos = randomPosition();

			if (checkForOverlap(tempPos)) {
				// Overlapping, try again
				continue;
			}

			var aTown : Town = new Town();
			aTown.pos = tempPos.clone();

			aTown.setPopulation( Main.rng.int(startPopMin, startPopMax) );

			towns.push( aTown );
		}
	}


	function draw() {
		trace('* World Draw');

		// Background
		var back : Sprite = new Sprite({
			name: 'scenery',
			pos: Luxe.screen.mid.clone(),
			size: new Vector(990, 668),
			centered: true,
			texture: Luxe.resources.texture('assets/sprites/Scene.png'),
			depth: -1
		});
	}

	
	// Remove sprites and such
	override function ondestroy() {
		trace('* World destroy');

		for (person in persons) {
			person.destroy();
		}

		for (town in towns) {
			town.destroy();
		}

		for (outpost in outposts) {
			outpost.destroy();
		}

		if (_scoreText != null) { _scoreText.destroy(); }
	}


	// Returns closest person to point
	public function findClosestPerson( fromPos : Vector ) : Person {

		var dist : Float = Math.POSITIVE_INFINITY;
		var closest : Person = null;

		for (p in persons) {

			var vecDist : Vector = p.pos.clone().subtract( fromPos );
			if ( vecDist.length < dist ) {
				dist = vecDist.length;
				closest = p;
			}

		}

		return closest;

	}


	// True = overlapping something within minRange
	public function checkForOverlap( aPos : Vector, minRange : Float = 80 ) : Bool {

		for (town in towns) {
			if (town.pos.clone().subtract(aPos).length < minRange) {
				return true;
			}
		}

		for (outpost in outposts) {
			if (outpost.pos.clone().subtract(aPos).length < minRange) {
				return true;
			}
		}

		return false;
	}


	// Random position within world
	public static function randomPosition() : Vector {
		return new Vector(Main.rng.int( 0 + wMargin, wWidth - wMargin), Main.rng.int( 0 + wMargin, wHeight - wMargin));
	}

}